#! python

# #####################################################################
# Copyright (c) 2017, Stanley Engle
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without 
# modification, are permitted provided that the following conditions 
# are met:
# 
# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
# AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY 
# WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# #####################################################################

# Standard library imports.
import argparse
import sys


# The version number of this script/module.
__version__ = '2'


def calculate(air_temp_c, rh, on_error = float('nan')):
    """Calculate the heat index.

    The heat index is calculated using the air temperature in degrees 
    Celsius and the relative humidity as a percentage (0.0 to 100.0).
    See:
    http://www.wpc.ncep.noaa.gov/html/heatindex_equation.shtml

    Parameters
    ----------
    air_temp_c : float
        The air temperature in degrees C.
    rh : float
        The relative humidity, as a percentage (0.0 to 100.0)
    on_error : optional, float('nan')
        The return value when an error occcurs during calculation.  May 
        be any type, but preferably a float.

    Returns
    -------
    float
        The heat index in degrees C.

    """
    try:

        at_f = (float(air_temp_c) * (9.0 / 5.0)) + 32.0
        rh_f = float(rh)

        heat_index = -42.379 + 2.04901523 * at_f + 10.14333127 * \
            rh_f - 0.22475541 * at_f * rh_f - 6.83783e-3 * \
            at_f ** 2 - 5.481717e-2 * rh_f ** 2 + \
            1.22874e-3 * at_f ** 2 * rh_f + 8.5282e-4 * at_f * \
            rh_f ** 2 - 1.99e-6 * at_f ** 2 * rh_f ** 2

        if (at_f <= 80.0) or (rh_f <= 40.0):
            heat_index = float(air_temp_c)

        else:
            heat_index = (heat_index - 32.0) * (5.0 / 9.0)

    except:
        heat_index = on_error

    return heat_index


def run(air_temp_c, rh, on_error = float('nan')):
    """Calls the calculate function.  See calculate docstring.

    """
    return calculate(air_temp_c, rh, on_error = on_error)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('air_temp_c', 
        type = float, 
        help = 'The air temperature in degrees Celsius.')
    parser.add_argument('rh', 
        type = float, 
        help = 'The relative humidity as a percentage (0.0 to 100.0).')

    # Parse the command line arguments
    args = parser.parse_args()

    # Call the run function.
    heat_index = run(args.air_temp_c, args.rh)

    # Print the output to standard out.
    sys.stdout.write(str(heat_index) + '\n')

    # Explicitly exit the script.
    sys.exit(0)


